---
elm:
  dependencies:
    avh4/elm-fifo: latest
    elm/browser: latest
    elm/core: latest
    elm/html: latest
    elm/json: latest
    elm/parser: latest
    elm/regex: latest
    elm/time: latest
    elm/url: latest
    elm/virtual-dom: latest
    elm-community/graph: latest
    elm-community/intdict: latest
    elm-community/list-extra: latest
    elm-community/maybe-extra: latest
    elm-community/string-extra: latest

  source-directories:
    - ../src
---

# Las guerras clon

> Fuente: Ejercios de Lógica Computacional
> (https://www.cs.us.es/~fsancho/?p=logica-informatica-2020-21)

Las guerras clon han comenzado. Durante el transcurso de una refriega, tres caballeros Jedi, Anakin, Obi Wan y Yoda, se encuentran con el conde Dooku. Utilizaremos el lenguaje proposicional A, O, Y para denotar que el correspondiente caballero participa en el combate, y G para denotar los Jedi han ganado.

Se sabe que:

- Para derrotar al conde Dooku deben participar al menos dos caballeros Jedi.
- El Conde Dooku gana cuando sólo participa un caballero.
- Si el Conde Dooku pierde entonces Anakin ha participado en el combate.

¿Es cierto que se puede deucir que si los Jedi ganan entonces Anakin y Obi Wan han participado en el combate?

## Solución

Los enunciados pueden formalizarse como:

- $G \rightarrow (A \wedge O) \vee (A \wedge Y) \vee (O \wedge Y)$

- $(A \wedge \neg O \wedge \neg Y) \vee (\neg A \wedge O \wedge \neg Y) \vee (\neg A \wedge \neg O \wedge Y) \rightarrow \neg G $

- $G \rightarrow A$

Y la consecuencia como

- $G \rightarrow A \wedge O$

Ahora, determinar la consistencia del conjunto es equivalente a determinar la satisfactibilidad de la conjunción de las fórmulas. Para lo que utilizaremos la FND. Haciendo uso de LogicUS:

```elm {l id="imports"}
import LogicUS.PL.SintaxSemantics exposing (..)
import LogicUS.PL.Clauses exposing (..)
import LogicUS.PL.DPLL exposing (..)

fplRead : String -> FormulaPL
fplRead = fplReadExtraction << fplReadFromString
```

Definimos las fórmulas:

```elm {l context="1" follows="imports"}
f1 : FormulaPL
f1 = fplRead "g -> (a & o ) | ( a & y ) | (o & y)"

f2 : FormulaPL
f2 = fplRead "a & ¬o & ¬y | ¬a & o & ¬y | ¬a & ¬o & y -> ¬g"

f3 : FormulaPL
f3 = fplRead "g -> a"

f4 : FormulaPL
f4 = fplRead "g -> a & o"

```

Entonces, determinar la consecuencia es equivalente a determinar la inconsistencia del conjunto $\{F_1, F_2, F_3, \neg F_4\}$

```elm {l r context="1"}
cs : ClausePLSet
cs = splToClauses [f1,f2,f3, Neg f4]
```

<br> Que corresponde a una lista de cláusulas correspondientes a listas de literales expresados como pares _(símbolo,signo)_. Representándolas:

^^^elm{m=("$ " ++ (csplToMathString cs) ++ " $") context="1"}^^^

De forma que omitiendo las que son subsumidas por otras:

^^^elm{m=("$ " ++ (csplToMathString <| csplRemoveSubsumedClauses cs) ++ " $") context="1"}^^^

De forma que aplicando el algoritmo dpll y representándolo en formato DOT

```elm {l r context="1"}
csTableau : String
csTableau =  dpll cs |> dpllTableauToDOT
```

Representándolo:

<center>

```dot
digraph G {
rankdir=TB
graph []
node [shape=box, color=black]
edge [dir=none, color=blue, fontcolor=blue]

0 -> 1 [label="g"]
1 -> 2 [label="a"]
2 -> 3 [label="¬ o"]
3 -> 4 [label="y"]

0 [label="{¬ g,o,y}, {a,¬ g}, {g}, {¬ a,¬ o}"]
1 [label="{o,y}, {a}, {¬ a,¬ o}"]
2 [label="{o,y}, {¬ o}"]
3 [label="{y}"]
4 [label="◯"]
}
```

</center>
