---
elm:
  dependencies:
    avh4/elm-fifo: latest
    elm/browser: latest
    elm/core: latest
    elm/html: latest
    elm/json: latest
    elm/parser: latest
    elm/regex: latest
    elm/time: latest
    elm/url: latest
    elm/virtual-dom: latest
    elm-community/graph: latest
    elm-community/intdict: latest
    elm-community/list-extra: latest
    elm-community/maybe-extra: latest
    elm-community/string-extra: latest

  source-directories:
    - ../src
---

# Oferta especial en Sortilegios Weasley

> Fuente: Ejercios de Lógica Computacional
> (https://www.cs.us.es/~fsancho/?p=logica-informatica-2020-21)

Sortilegios Weasley, también conocida como Weasley & Weasley, es una tienda de artículos de bromas ubicada en el Callejón Diagon propiedad de los gemelos Fred y George Weasley. Entre otros productos en la tienda disponen de _Sombreros acéfalos_ (A), _Bombones desmayo_(B), _Caramelos longuilinguos_ (C), _Fantasías patentadas_ (F), _Libro mordedor_ (L) y _Orejas extensibles_ (O).

Los hermanos Weasley han sacado una caja sorpresa a la venta de la que han hecho varios carteles en los que se puede leer:

- Si la caja contiene sombreros acéfalos y bombones desmayo entonces también tiene caramelos longuilinguos.

- ¡Que no te engañen! En la caja nada de fantasías patentadas, lo que sí tiene es un libro mordedor y sombreros acéfalos.

- ¿Y si tuviera sombreros acéfalos y orejas extensibles? Entonces, tendría también fantasías patentadas.

- ¿Qué tal un libro mordedor? En tal caso deberían incluirse unos bombones desmayo como compensación.

- No somos partidarios de incluir sombreros acéfalos y libro mordedor, pero en tal caso también incluiríamos unas orejas extensibles.

Hermione, que es muy avispada, concluyó que los hermanos no habían dicho la verdad en todos los carteles, sin necesidad de recurrir a la maldición _Cruciatus_. ¿Estaba en lo cierto?

## Solución

Los enunciados de los carteles pueden formalizarse en el conjunto

$$ \{A \wedge B \rightarrow C, \, ¬F \wedge L \wedge A, A \wedge O \rightarrow F, L \rightarrow B, A \wedge L \rightarrow O\} $$

Demostremos que el conjunto es inconsistente utilizando distintas estrategias de resolución. Nótese que corresponde a un conjunto de cláusulas de Horn, por lo que todas las estrategias expuestas son refutacionalmente completas.

Expresemos el conjunto como un conjunto de cláusulas:

```elm {l id="imports"}
import LogicUS.PL.SintaxSemantics exposing (..)
import LogicUS.PL.Clauses exposing (..)
import LogicUS.PL.Resolution exposing (..)


fplRead : String -> FormulaPL
fplRead = fplReadExtraction << fplReadFromString
```

Definimos las fórmulas:

```elm {l context="1" follows="imports"}
f1 : FormulaPL
f1 = fplRead "a & b -> c"

f2 : FormulaPL
f2 = fplRead "¬f & l & a"

f3 : FormulaPL
f3 = fplRead "a & o -> f"

f4 : FormulaPL
f4 = fplRead "l -> b"

f5 : FormulaPL
f5 = fplRead "a & l -> o"

```

Entonces, el conjunto de cláusulas correspondería a:

```elm {l context="1"}
cs : ClausePLSet
cs = splToClauses [f1,f2,f3, f4, f5]
```

^^^elm{m=("$$ " ++ (csplToMathString cs) ++ " $$") context="1"}^^^

<br>

#### Resolución regular

```elm {l r="Tuple.first cs_reg" context="1"}
cs_reg : (Bool, List (List ClausePL))
cs_reg = csplRegularResolution (csplSymbols cs) cs
```

<br> De forma que el proceso correspondería a :

^^^elm{m=("$$ " ++ (cs_reg |> Tuple.second |> resolutionProcessListToMathString) ++ " $$") context="1"}^^^

En efecto, se llega a la cláusula vacía luego el conjunto es insatisfactible.

#### Resolución lineal SCF

```elm {l r="Tuple.first cs_lin" context="1"}
cs_lin : (Bool, ResolutionTableau)
cs_lin = csplSCFLinearResolution cs
```

<br> De forma que el proceso correspondería a :

```dot
digraph G {
rankdir=TB
graph []
node [shape=box, color=black]
edge [color=blue, fontcolor=blue]

1 -> 9 [label="f"]
2 -> 13 [label="¬ l"]
3 -> 10 [label="¬ a"]
3 -> 12 [label="¬ a"]
6 -> 9 [label="¬ f"]
7 -> 11 [label="¬ o"]
9 -> 10 [label="a"]
10 -> 11 [label="o"]
11 -> 12 [label="a"]
12 -> 13 [label="l"]

1 [label="{¬ f}"]
2 [label="{l}"]
3 [label="{a}"]
6 [label="{¬ a,f,¬ o}"]
7 [label="{¬ a,¬ l,o}"]
9 [label="{¬ a,¬ o}"]
10 [label="{¬ o}"]
11 [label="{¬ a,¬ l}"]
12 [label="{¬ l}"]
13 [label="□"]
14 [label="{¬ a,¬ b,c}"]
15 [label="{b,¬ l}"]

{rank=same; 1;2;3;6;7;14;15;}
}
```

En efecto, se llega a la cláusula vacía luego el conjunto es insatisfactible. Nótese que también es por entradas, unitaria y negativa. Aunque existan otras, ejcutemos la unitaria por ejemplo:

#### Resolución unitaria SCF

<!-- m="resolutionTableauToDOT <| Tuple.second <| cs_unit" -->

```elm {l r="Tuple.first cs_unit"  context="1"}
cs_unit : (Bool, ResolutionTableau)
cs_unit = csplSCFUnitaryResolution cs
```

<br> De forma que el proceso correspondería a :

```dot
digraph G {
rankdir=TB
graph []
node [shape=box, color=black]
edge [color=blue, fontcolor=blue]

1 -> 9 [label="f"]
2 -> 12 [label="¬ l"]
3 -> 14 [label="¬ a"]
3 -> 15 [label="¬ a"]
6 -> 9 [label="¬ f"]
7 -> 12 [label="l"]
9 -> 15 [label="a"]
12 -> 14 [label="a"]
14 -> 20 [label="¬ o"]
15 -> 20 [label="o"]

1 [label="{¬ f}"]
2 [label="{l}"]
3 [label="{a}"]
6 [label="{¬ a,f,¬ o}"]
7 [label="{¬ a,¬ l,o}"]
9 [label="{¬ a,¬ o}"]
12 [label="{¬ a,o}"]
14 [label="{o}"]
15 [label="{¬ o}"]
20 [label="□"]
21 [label="{¬ a,¬ b,c}"]
22 [label="{b,¬ l}"]

{rank=same; 1;2;3;6;7;21;22;}
}
```

Sale otro proceso de resolución, pero en este caso no es ni lineal ni por entradas.

Veamos finalmente una resolución positiva:

#### Resolución positiva SCF

<!-- m="resolutionTableauToDOT <| Tuple.second <| cs_pos" -->

```elm {l r="Tuple.first cs_pos" context="1"}
cs_pos : (Bool, ResolutionTableau)
cs_pos = csplSCFPositiveResolution cs
```

<br> De forma que el proceso correspondería a :

```dot
digraph G {
rankdir=TB
graph []
node [shape=box, color=black]
edge [color=blue, fontcolor=blue]

1 -> 22 [label="f"]
2 -> 11 [label="¬ l"]
3 -> 13 [label="¬ a"]
3 -> 14 [label="¬ a"]
6 -> 14 [label="a"]
7 -> 11 [label="l"]
11 -> 13 [label="a"]
13 -> 19 [label="¬ o"]
14 -> 19 [label="o"]
19 -> 22 [label="¬ f"]

1 [label="{¬ f}"]
2 [label="{l}"]
3 [label="{a}"]
6 [label="{¬ a,f,¬ o}"]
7 [label="{¬ a,¬ l,o}"]
11 [label="{¬ a,o}"]
13 [label="{o}"]
14 [label="{f,¬ o}"]
19 [label="{f}"]
22 [label="□"]
23 [label="{¬ a,¬ b,c}"]
24 [label="{b,¬ l}"]

{rank=same; 1;2;3;6;7;23;24;}
}
```
